
#include "FramesFromFileSourceFilter.h"

#define VIDEO_WIDTH		352
#define VIDEO_HEIGHT	288

using namespace std;

unsigned __stdcall FramesFromFileStreamThread(LPVOID pvParam)
{
	struct stat results;
	int fileSize;
	int i;
	int count = 0;

	FramesFromFilePin* pin = (FramesFromFilePin*)pvParam;

	Wow64DisableWow64FsRedirection(NULL);
	HMODULE hModule = GetModuleHandleA("FramesFromFileSourceFilter.ax");
	char file[MAX_PATH];
	GetModuleFileNameA(hModule, file, MAX_PATH);
	
	for(i = strlen(file); i >= 0; i--) {
		if(file[i] == L'\\') {
			count++;
		}
		if(count == 2)
			break;
	}
	file[i] = 0;

	std::string path = file;
	path += "\\H264Frames\\";
	std::string filename;

	for(int frameNumber = 1; frameNumber <= 1178; frameNumber++) {
		std::stringstream ss;
		ss << frameNumber;
		std::string zeroPaddedNumber;
		ss >> zeroPaddedNumber;

		while(zeroPaddedNumber.size() < 4) {
			zeroPaddedNumber = "0" + zeroPaddedNumber;
		}

		filename = path + "Stream_" + zeroPaddedNumber + ".dat";

		if(stat(filename.c_str(), &results) != 0) {
			continue;
		}

		fileSize = results.st_size;
		pin->m_bufferSize = fileSize;
		pin->m_buffer = (BYTE*)malloc(pin->m_bufferSize);

		try {
			ifstream myFile (filename.c_str(), ios::in | ios::binary);
			myFile.read((char *)pin->m_buffer, fileSize);
			myFile.close();

			if(fileSize <= 0) {
				continue;			
			}

			Sleep(83);

			pin->isFrameReady = true;
		} catch(...) {
			MessageBoxA(GetDesktopWindow(), "Triggered", "Error", MB_OK);
		}
	}

	return 0;
}

const AMOVIESETUP_MEDIATYPE sudOpPinTypes =
{
	&MEDIATYPE_Video,       // Major type
	&MEDIASUBTYPE_NULL      // Minor type
};


/**********************************************
*
*  FramesFromFilePin Class
*
**********************************************/

FramesFromFilePin::FramesFromFilePin(HRESULT *phr, CSource *pFilter)
: CSourceStream(NAME("Frames From File Source Bitmap"), phr, pFilter, L"Out"),
isFrameReady(false)
{
	m_bmpInfo.biSize = sizeof(BITMAPINFOHEADER);
	m_bmpInfo.biCompression = DWORD('xvid');
	m_bmpInfo.biBitCount = 24;
	m_bmpInfo.biWidth = VIDEO_WIDTH;
	m_bmpInfo.biHeight = VIDEO_HEIGHT;
	m_bmpInfo.biSize = VIDEO_WIDTH * VIDEO_HEIGHT * 3;
	m_bmpInfo.biSizeImage = GetBitmapSize(&m_bmpInfo);
	m_bmpInfo.biPlanes = 1;
	m_bmpInfo.biClrImportant = 0;
	m_bmpInfo.biClrUsed = 0;
	m_bmpInfo.biXPelsPerMeter = 0;
	m_bmpInfo.biYPelsPerMeter = 0;

	*phr = S_OK;
}


FramesFromFilePin::~FramesFromFilePin()
{   
}

HRESULT FramesFromFilePin::OnThreadCreate() {
	m_hFramesFromFileStreamThreadHandle = (HANDLE)_beginthreadex(NULL, 0, &FramesFromFileStreamThread, (void*)this, 0, &m_dwThreadID);

	return S_OK;
}

HRESULT FramesFromFilePin::OnThreadDestroy() {
	CloseHandle(m_hFramesFromFileStreamThreadHandle);

	return S_OK;
}


HRESULT FramesFromFilePin::GetMediaType(CMediaType *pMediaType)
{
	CAutoLock cAutoLock(m_pFilter->pStateLock());

	CheckPointer(pMediaType, E_POINTER);

	VIDEOINFOHEADER* pvi = (VIDEOINFOHEADER*)pMediaType->AllocFormatBuffer(sizeof(VIDEOINFOHEADER));
	if (pvi == 0) 
		return(E_OUTOFMEMORY);

	ZeroMemory(pvi, pMediaType->cbFormat);   

	pvi->bmiHeader = m_bmpInfo;
	pvi->bmiHeader.biSizeImage = GetBitmapSize(&pvi->bmiHeader);
	SetRect(&pvi->rcSource, 0, 0, VIDEO_WIDTH, VIDEO_HEIGHT);
	pvi->rcTarget = pvi->rcSource;

	SetRectEmpty(&(pvi->rcSource));
	SetRectEmpty(&(pvi->rcTarget));

	pMediaType->SetType(&MEDIATYPE_Video);
	pMediaType->SetFormatType(&FORMAT_VideoInfo);
	pMediaType->SetTemporalCompression(FALSE);

	const GUID SubTypeGUID = __uuidof(Subtype_H264);
	pMediaType->SetSubtype(&SubTypeGUID);
	pMediaType->SetSampleSize(pvi->bmiHeader.biSizeImage);

	return S_OK;
}

HRESULT FramesFromFilePin::DecideBufferSize(IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *pRequest)
{
	HRESULT hr;
	CAutoLock cAutoLock(m_pFilter->pStateLock());

	CheckPointer(pAlloc, E_POINTER);
	CheckPointer(pRequest, E_POINTER);

	VIDEOINFOHEADER *pvi = (VIDEOINFOHEADER*) m_mt.Format();

	if (pRequest->cBuffers == 0)
	{
		pRequest->cBuffers = 2;
	}
	pRequest->cbBuffer = pvi->bmiHeader.biSizeImage;

	ALLOCATOR_PROPERTIES Actual;
	hr = pAlloc->SetProperties(pRequest, &Actual);
	if (FAILED(hr)) 
	{
		return hr;
	}

	if (Actual.cbBuffer < pRequest->cbBuffer) 
	{
		return E_FAIL;
	}

	return S_OK;
}

HRESULT FramesFromFilePin::FillBuffer(IMediaSample *pSample) {
	CAutoLock cAutoLock(m_pLock);
	HRESULT hr;

	BYTE* pData = NULL;

	hr = pSample->GetPointer(&pData);
	if(FAILED(hr)) {
		pSample->Release();
		return hr;
	}

	memcpy(pData, m_buffer, m_bufferSize);

	free(m_buffer);

	hr = pSample->SetActualDataLength(m_bufferSize);
	if(FAILED(hr)) {
		pSample->Release();
		return hr;
	}

	hr = pSample->SetSyncPoint(TRUE);
	if(FAILED(hr)) {
		pSample->Release();
		return hr;
	}

	return S_OK;
}

HRESULT FramesFromFilePin::DoBufferProcessingLoop() {
	Command com;
	REFERENCE_TIME rtNow = 0L;
	REFERENCE_TIME rtAdvise = 0L;

	OnThreadStartPlay();

	do {
		while (!CheckRequest(&com)) {
			if(isFrameReady) {
				isFrameReady = false;
				IMediaSample *pSample;

				HRESULT hr = GetDeliveryBuffer(&pSample,NULL,NULL,FALSE);
				if (FAILED(hr)) {
					continue;
				}

				hr = FillBuffer(pSample);

				if (hr == S_OK) {
					Deliver(pSample);
				} else if (hr == S_FALSE) {
					pSample->Release();
					DeliverEndOfStream();
					return S_OK;
				} else {
					// empty
				}

				pSample->Release();
			}
		}

		if (com == CMD_RUN || com == CMD_PAUSE)
			com = GetRequest();
		else if (com != CMD_STOP) {
			// empty
		}
	} while (com != CMD_STOP);

	return S_FALSE;
}

/**********************************************
*
*  FramesFromFileSourceFilter Class
*
**********************************************/

FramesFromFileSourceFilter::FramesFromFileSourceFilter(IUnknown *pUnk, HRESULT *phr)
: CSource(NAME("FramesFromFileSourceBitmap"), pUnk, CLSID_FramesFromFileSourceFilter) {
	m_pPin = new FramesFromFilePin(phr, this);

	if (phr) {
		if (m_pPin == NULL) {
			*phr = E_OUTOFMEMORY;
		} else {
			*phr = S_OK;
		}
	}
}


FramesFromFileSourceFilter::~FramesFromFileSourceFilter() {
	delete m_pPin;
}

CUnknown * WINAPI FramesFromFileSourceFilter::CreateInstance(IUnknown *pUnk, HRESULT *phr) {
	FramesFromFileSourceFilter *pNewFilter = new FramesFromFileSourceFilter(pUnk, phr);

	if (phr) {
		if (pNewFilter == NULL) {
			*phr = E_OUTOFMEMORY;
		} else {
			*phr = S_OK;
		}
	}

	return pNewFilter;
}

STDMETHODIMP FramesFromFileSourceFilter::NonDelegatingQueryInterface( REFIID riid, void **ppv ) {
	return CSource::NonDelegatingQueryInterface(riid, ppv);
}
